/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package beaconconfig;

import cricketdaemon.clientlib.ServerBroker;
import cricketdaemon.clientlib.data.BeaconRecord;
import java.awt.*;
import java.awt.event.*;
import java.util.*;

class Beacon extends Vertex
{
    public BeaconRecord beaconRecord;
    private ServerBroker server;

    public Beacon(BeaconRecord br, ServerBroker s)
    {
	super(br.pos.x, br.pos.y);
	super.beacons.add(this);
	beaconRecord = br;
	server = s;
    }
    
    public void mousePressed(double x, double y, int modifier)
    {
	//if((modifier & MouseEvent.CTRL_MASK) > 0) {
	
	//}
    }

    public void draw(Graphics g, DrawingArea drawArea) 
    {
	super.draw(g, drawArea);

	/*
	  System.out.println("DrawBeacon: "+beaconRecord.space+" "+
	  drawArea.isAnnulusMode()+" "+
	  (beaconRecord != null)+" "+
	  (beaconRecord.distStat != null));
	*/
			 
	if(drawArea.isAnnulusMode() && beaconRecord != null && beaconRecord.distStat != null) {
	    double radius = SolverPanel.K_NOMINAL*beaconRecord.distStat.mean;
	    double endRadius = SolverPanel.K_NOMINAL*(beaconRecord.distStat.mean + beaconRecord.distStat.stddev);
	    double height = SolverPanel.getCurrentPos().z;
	    
	    radius = SolverPanel.diffsqr(radius, height);
	    if(radius < 0)
		radius = 0;
	    radius = Math.sqrt(radius);

	    endRadius = SolverPanel.diffsqr(endRadius, height);
	    if(endRadius < 0)
		endRadius = 0;
	    endRadius = Math.sqrt(endRadius);
	    
	    Vertex p = drawArea.transform(this.x-radius, this.y+radius);
	    Vertex q = drawArea.transform(this.x+radius, this.y-radius);
	    
	    g.drawOval(Math.round((float)p.x), Math.round((float)p.y), 
		       Math.round((float)Math.abs(q.x-p.x)), Math.round((float)Math.abs(q.y-p.y)));
	    
	    
	    Vertex pp = drawArea.transform(this.x-endRadius, this.y+endRadius);
	    //Vertex qq = drawArea.transform(this.x+endRadius, this.y-endRadius);
	    
	    int numSteps = Math.abs(Math.round((float)pp.x) - Math.round((float)p.x));
	    
	    while(numSteps > 0) {
		g.drawOval(Math.round((float)p.x)-numSteps, Math.round((float)p.y)-numSteps, 
			   Math.round((float)Math.abs(q.x-p.x))+2*numSteps, Math.round((float)Math.abs(q.y-p.y))+2*numSteps);
		numSteps--;
	    }
	}
	g.drawString(beaconRecord.space, 
		     Math.round((float)transformed_xyz.x)+2, 
		     Math.round((float)(transformed_xyz.y))-3);
    }

    public void mouseDragged(double x, double y, Vertex anchor, 
			     int modifier) 
    {
	super.mouseDragged(x, y, anchor, modifier);
	beaconRecord.pos.x = x;
	beaconRecord.pos.y = y;
    }

    public void mouseReleased(double x, double y)
    {
	//System.out.println("registering beacon coords: "+beaconRecord.space+" "+
	//beaconRecord.id+" "+beaconRecord.pos.x+" "+beaconRecord.pos.y);
	server.setBeaconTableEntry(beaconRecord.space, beaconRecord.id.intValue(),
				   beaconRecord.pos.x, beaconRecord.pos.y, 0);
    }

    public static BeaconRecord findBeacon(BeaconRecord br) {
	Iterator it = beacons.iterator();
	while(it.hasNext()) {
	    BeaconRecord beacon = ((Beacon) it.next()).beaconRecord;
	    if(beacon.equals(br)) {
		return beacon;
	    }
	}
	return null;
    }

    public String toString() {
	return beaconRecord.toString();
    }
}
