/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.reader;

import cricketdaemon.clientlib.data.Position;
import cricketdaemon.Util;
import java.util.*;

/**
 * 3D coordinate object used as a primitive type of
 * the CricketClient beacon packet.
 *
 * @author Allen Miu
 */
public class CartesianCoord extends Position
{
    public CartesianCoord(double x, double y, double z)
    {
	super(x, y, z);
    }

    public static CartesianCoord parseCoord(String s)
    {
	StringTokenizer st = new StringTokenizer(s, " \t\n\r\f");
	st.nextToken(); // pop (
	return parseCoord(st);
    }

    public static CartesianCoord parseCoord(StringTokenizer st)
    {
	try {
	    double x = Double.parseDouble(st.nextToken());
	    double y = Double.parseDouble(st.nextToken());
	    double z = Double.parseDouble(st.nextToken());

	    return new CartesianCoord(x, y, z);
	}
	catch(NoSuchElementException e) {
	    Util.log("CartesianCoord::parseCoord() bad input");
	    return null;
	}
	catch(NumberFormatException ne) {
	    Util.log("CartesianCoord::parseCoord() bad input");
	    return null;
	}
    }
}
