/*    
       This software may be used and distributed according to the terms of
       the GNU General Public License (GPL), incorporated herein by
       reference.  Drivers based on this skeleton fall under the GPL and
       must retain the authorship (implicit copyright) notice.
 
       This program is distributed in the hope that it will be useful, but
       WITHOUT ANY WARRANTY; without even the implied warranty of
       MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
       General Public License for more details.
 */
package cricketdaemon.server;

import cricketdaemon.Util;
import java.net.*;
import java.io.*;

/**
 * A thread that listens to incoming registrations.
 *
 * @author Allen Miu
 */
public class ServerThread extends Thread
{
    public static final String REGISTER_TOKEN    = "register";
    public static final String BEACONTABLE_TOKEN = "beacontable";
    public static final String POSITION_TOKEN    = "position";

    ServerConnListener scl;
    ServerSocket s;
    
    ServerThread(ServerConnListener listener, int port) 
    {
	scl = listener;
	try {
	    s = new ServerSocket(port);
	    s.setSoTimeout(0); // enable blocking
	}
	catch (IOException e) {
	    Util.assert_nonjava(false, "error opening port "+port);
	}
    }
    
    public void run()
    {
	//this should just register each client
	byte[] buf = new byte[64];
	while(true) {
	    try {
		Socket newSock = s.accept();
		newSock.setTcpNoDelay(true);
		System.out.println("CricketDaemon accepting new connection from "+
				   newSock.getRemoteSocketAddress());
		scl.registerClient(newSock); 
	    }
	    catch (IOException io) {
		io.printStackTrace();
		Util.assert_nonjava(false,"ServerThread::run() ioexception");
	    }
	}
	//s.close(); never reached 
    }
}
